@@ -2,12 +2,12 @@
  * Defines the interface for the Accessor class.
  *
  * \author Martin F. Krafft <krafft@ailab.ch>
- * \date $Date: 2004/05/15 22:49:29 $
- * \version $Revision: 1.2 $
+ * \date $Date: 2004/11/15 17:40:32 $
+ * \version $Revision: 1.5 $
  */
 
-#ifndef __INCLUDE_KDTREE_ACCESSOR_HPP__
-#define __INCLUDE_KDTREE_ACCESSOR_HPP__
+#ifndef INCLUDE_KDTREE_ACCESSOR_HPP
+#define INCLUDE_KDTREE_ACCESSOR_HPP
 
 #include <cstddef>
 
@@ -29,14 +29,14 @@ namespace KDTree
       typedef _Val value_type;
       typedef typename value_type::value_type subvalue_type;
       
-      inline subvalue_type
+      subvalue_type
       operator()(value_type const& V, size_t const N) const throw ()
       {
         return V[N];
       }
     };
 
-}; // namespace KDTree
+} // namespace KDTree
 
 #endif // include guard
 
