@@ -2,12 +2,12 @@
  * Defines the interface for the KDTree class.
  *
  * \author Martin F. Krafft <krafft@ailab.ch>
- * \date $Date: 2004/05/15 22:49:29 $
- * \version $Revision: 1.3 $
+ * \date $Date: 2004/11/15 17:40:32 $
+ * \version $Revision: 1.9 $
  */
 
-#ifndef __INCLUDE_KDTREE_KDTREE_HPP__
-#define __INCLUDE_KDTREE_KDTREE_HPP__
+#ifndef INCLUDE_KDTREE_KDTREE_HPP
+#define INCLUDE_KDTREE_KDTREE_HPP
 
 #include <vector>
 
@@ -28,15 +28,39 @@ namespace KDTree
 {
 
 #ifdef KDTREE_DEFINE_OSTREAM_OPERATORS
+
   template <size_t const __K, typename _Val,
             typename _Acc, typename _Cmp, typename _Alloc>
-    class KDTree;
+    std::ostream&
+    operator<<(std::ostream& __out,
+               KDTree<__K, _Val, _Acc, _Cmp, _Alloc> const& __T) throw ()
+    {
+      __out << "meta node:   " << *__T._M_header << std::endl;
+
+      if (__T.empty())
+        return __out << "[empty " << __K << "d-tree " << &__T << "]";
+
+      __out << "nodes total: " << __T.size() << std::endl;
+      __out << "dimensions:  " << __K << std::endl;
+
+      typedef KDTree<__K, _Val, _Acc, _Cmp, _Alloc> _Tree;
+      typedef typename _Tree::_Link_type _Link_type;
+
+      std::stack<_Link_type> s;
+      s.push(__T._M_root());
+      
+      while (!s.empty())
+        {
+          _Link_type n = s.top();
+          s.pop();
+          std::cout << *n << std::endl;
+          if (_Tree::_S_left(n)) s.push(_Tree::_S_left(n));
+          if (_Tree::_S_right(n)) s.push(_Tree::_S_right(n));
+        }
+
+      return __out;
+    }
 
-  template <typename _Char, typename _Traits, size_t const __K, typename _Val,
-            typename _Acc, typename _Cmp, typename _Alloc>
-    std::basic_ostream<_Char, _Traits>&
-    operator<<(std::basic_ostream<_Char, _Traits>,
-               KDTree<__K, _Val, _Acc, _Cmp, _Alloc> const&) throw ();
 #endif
 
   template <size_t const __K, typename _Val,
@@ -47,8 +71,8 @@ namespace KDTree
     {
     protected:
       typedef _Alloc allocator_type;
-      typedef _Alloc_base<_Val, _Alloc> _Alloc_base;
-      typedef typename _Alloc_base::allocator_type allocator_type;
+      typedef _Alloc_base<_Val, _Alloc> _Base;
+      typedef typename _Base::allocator_type allocator_type;
 
       typedef _Node_base* _Base_ptr;
       typedef _Node<_Val>* _Link_type;
@@ -68,148 +92,155 @@ namespace KDTree
       typedef size_t size_type;
       typedef ptrdiff_t difference_type;
 
-      inline
-      KDTree() throw ()
-        : _Alloc_base(allocator_type()), _M_meta(_M_new_node()), _M_count(0)
+      KDTree(const allocator_type& __a = allocator_type()) throw ()
+        : _Base(__a), _M_header(_Base::_M_allocate_node()), _M_count(0)
       {
+         _M_empty_initialise();
+      }
+
+      KDTree(const KDTree& __x) throw ()
+         : _Base(__x.get_allocator()), _M_header(_Base::_M_allocate_node()), _M_count(0)
+      {
+         _M_empty_initialise();
+         this->insert(begin(), __x.begin(), __x.end()); 
+      }
+
+      template<typename _InputIterator>
+        KDTree(_InputIterator __first, _InputIterator __last,
+             const allocator_type& __a = allocator_type()) throw ()
+        : _Base(__a), _M_header(_Base::_M_allocate_node()), _M_count(0)
+      {
+         _M_empty_initialise();
+         this->insert(begin(), __first, __last); 
+      }
+
+      KDTree&
+      operator=(const KDTree& __x)
+      {
+         if (this != &__x)
+         {
+            this->clear();
+            this->insert(begin(),__x.begin(),__x.end());
+         }
+         return *this;
       }
 
-      inline
       ~KDTree() throw ()
       {
-        clear();
-        _M_delete_node(_M_meta);
+        this->clear();
+        _M_deallocate_node(_M_header);
       }
 
-      inline allocator_type
+      allocator_type
       get_allocator() const 
       { 
-        return _Alloc_base::get_allocator(); 
+        return _Base::get_allocator(); 
       }
       
-      inline size_type
+      size_type
       size() const
       {
         return _M_count;
       }
       
-      inline size_type
+      size_type
       max_size() const
       { 
         return size_type(-1); 
       }
 
-      inline bool
+      bool
       empty() const
       {
         return this->size() == 0;
       }
 
-      inline void
+      void
       clear()
       {
         _M_erase_subtree(_M_root());
-        _M_leftmost() = _M_rightmost() = _M_meta;
+        _M_leftmost() = _M_rightmost() = _M_header;
         _M_root() = NULL;
         _M_count = 0;
       }
 
 //      typedef _Iterator<_Val, reference, pointer> iterator;
       typedef _Iterator<_Val, const_reference, const_pointer> const_iterator;
+
+      // there is no such thing as a mutable iterator,
+      // but we need to define one anyway.
+      typedef const_iterator iterator;
+
 //      typedef std::reverse_iterator<iterator> reverse_iterator;
       // TODO: const_reverse_iterators are broken
       // see http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=244894
       typedef std::reverse_iterator<const_iterator> const_reverse_iterator;
+      typedef const_reverse_iterator reverse_iterator;
 
-      const_iterator
-      begin() const 
-      { 
-        return const_iterator(_M_leftmost()); 
-      }
+      iterator begin() const { return iterator(_M_leftmost()); }
+      iterator end() const { return iterator(_M_header); }
+      reverse_iterator rbegin() const { return reverse_iterator(end()); }
+      reverse_iterator rend() const { return reverse_iterator(begin()); }
 
-/* we must not provide access to mutable members...
       iterator
-      begin()
-      { 
-        return iterator(_M_leftmost()); 
-      }
-*/
-
-      const_iterator
-      end() const
-      { 
-        return const_iterator(_M_meta);
+      insert(iterator /* ignored */, const_reference __V) throw (std::bad_alloc)
+      {
+         return this->insert(__V);
       }
 
-/* we must not provide access to mutable members...
       iterator
-      end()
-      { 
-        return iterator(_M_meta);
-      }
-*/
-
-      const_reverse_iterator
-      rbegin() const
-      { 
-        return const_reverse_iterator(end()); 
-      }
-
-/* we must not provide access to mutable members...
-      reverse_iterator
-      rbegin()
-      { 
-        return reverse_iterator(end()); 
-      }
-*/
-
-      const_reverse_iterator
-      rend() const
-      { 
-        return const_reverse_iterator(end()); 
-      }
-
-/* we must not provide access to mutable members...
-      reverse_iterator
-      rend()
-      { 
-        return reverse_iterator(end()); 
-      }
-*/
-
-      inline const_iterator
       insert(const_reference __V) throw (std::bad_alloc)
       {
         if (!_M_root())
           {
-            _Link_type __n = _M_new_node(__V, _M_meta); ++_M_count;
+            _Link_type __n = _M_new_node(__V, _M_header); 
+            ++_M_count;
             _M_root() = _M_leftmost() = _M_rightmost() = __n;
-            return const_iterator(__n);
+            return iterator(__n);
           }
         return _M_insert(_M_root(), __V, 0);
       }
 
-      inline void
-      erase(const_reference __V) throw ()
+      template <class _InputIterator>
+      void insert(_InputIterator __first, _InputIterator __last) {
+         for (; __first != __last; ++__first)
+            this->insert(*__first);
+      }
+      
+      void
+      insert(iterator __pos, size_type __n, const value_type& __x)
       {
+        for (; __n > 0; --__n)
+          this->insert(__pos, __x);
+      }
+
+      template<typename _InputIterator>
+      void
+      insert(iterator __pos, _InputIterator __first, _InputIterator __last) {
+         for (; __first != __last; ++__first)
+            this->insert(__pos, *__first);
+      }
+
+      void
+      erase(const_reference __V) throw () {
         this->erase(this->find(__V));
       }
 
-      inline void
-      erase(const_iterator const& __IT) throw ()
+      void
+      erase(iterator const& __IT) throw ()
       {
         if (__IT == this->end()) return;
         _Link_type const __N = (_Link_type const) __IT._M_node;
         _Link_type __n = __N;
         size_t __l = 0;
-        while ((__n = _S_parent(__n)) != _M_meta) ++__l;
+        while ((__n = _S_parent(__n)) != _M_header) ++__l;
         _M_erase(__N, __l);
         _M_delete_node(__N); --_M_count;
       }
 
 /* this does not work since erasure changes sort order
-      inline void
-      erase(const_iterator __A, const_iterator const& __B) throw ()
+      void
+      erase(iterator __A, iterator const& __B) throw ()
       {
         if (0 && __A == this->begin() && __B == this->end())
           {
@@ -223,19 +254,42 @@ namespace KDTree
       }
 */
 
-      inline const_iterator
+      const_iterator
       find(const_reference __V) const throw ()
       {
         if (!_M_root()) return this->end();
         return _M_find(_M_root(), __V, 0);
       }
 
+      size_t
+        count_within_range(const_reference __V, subvalue_type const __R) const throw ()
+        {
+          if (!_M_root()) return 0;
+          _Region __region;
+          _Acc __acc;
+          for (size_t __i = 0; __i < __K; ++__i) {
+            __region._M_low_bounds[__i] = __acc(__V, __i) - __R;
+            __region._M_high_bounds[__i] = __acc(__V, __i) + __R;
+          }
+          return this->count_within_range(__region);
+        } 
+
+      size_t
+        count_within_range(_Region const& __REGION) const throw ()
+        {
+          if (!_M_root()) return 0;
+
+          _Region __bounds(__REGION);
+          return _M_count_within_range(_M_root(),
+                               __REGION, __bounds, 0);
+        }
+
       template <typename _OutputIterator>
-        inline size_t
+        _OutputIterator
         find_within_range(const_reference __V, subvalue_type const __R,
                           _OutputIterator __out) const throw ()
         {
-          if (!_M_root()) return 0;
+          if (!_M_root()) return __out;
           _Region __region;
           _Acc __acc;
           for (size_t __i = 0; __i < __K; ++__i) {
@@ -246,50 +300,60 @@ namespace KDTree
         } 
 
       template <typename _OutputIterator>
-        inline size_t
+        _OutputIterator
         find_within_range(_Region const& __REGION,
                           _OutputIterator __out) const throw ()
         {
-          size_t __cnt = 0;
           if (_M_root())
             {
               _Region __bounds(__REGION);
-              _M_find_within_range(__out, __cnt, _M_root(),
+              __out = _M_find_within_range(__out, _M_root(),
                                    __REGION, __bounds, 0);
             }
-          return __cnt;
+          return __out;
         }
 
-      inline void
+      void
       optimise()
       {
-        std::vector<value_type> __v;
-        std::copy(this->begin(), this->end(), std::back_inserter(__v));
-        clear();
+        std::vector<value_type> __v(this->begin(),this->end());
+        this->clear();
         _M_optimise(__v.begin(), __v.end(), 0);
       }
 
+      inline void
+      optimize()
+      { // cater for people who cannot spell :)
+        optimise();
+      }
+
     protected:
 
-      inline const_iterator
+      void _M_empty_initialise()
+      {
+        _M_leftmost() = _M_rightmost() = _M_header;
+        _M_root() = NULL;
+      }
+
+      iterator
       _M_insert_left(_Link_type __N, const_reference __V)
       {
         _S_left(__N) = _M_new_node(__V); ++_M_count;
         _S_left(__N)->_M_parent = __N;
         if (__N == _M_leftmost()) _M_leftmost() = _S_left(__N);
-        return const_iterator(_S_left(__N));
+        return iterator(_S_left(__N));
       }
 
-      inline const_iterator
+      iterator
       _M_insert_right(_Link_type __N, const_reference __V)
       {
         _S_right(__N) = _M_new_node(__V); ++_M_count;
         _S_right(__N)->_M_parent = __N;
         if (__N == _M_rightmost()) _M_rightmost() = _S_right(__N);
-        return const_iterator(_S_right(__N));
+        return iterator(_S_right(__N));
       }
 
-      inline const_iterator
+      iterator
       _M_insert(_Link_type __N, const_reference __V,
              size_t const __L) throw (std::bad_alloc)
       {
@@ -307,13 +371,13 @@ namespace KDTree
           }
       }
 
-      inline _Link_type
+      _Link_type
       _M_erase(_Link_type const __N, size_t const __L) throw ()
       {
         return _M_erase_replace(__N, _M_get_erase_replacement(__N, __L));
       }
 
-      inline _Link_type
+      _Link_type
       _M_get_erase_replacement(_Link_type const __N,
                                size_t const __L) throw ()
       {
@@ -339,7 +403,7 @@ namespace KDTree
         return __ret;
       }
 
-      inline _Link_type
+      _Link_type
       _M_erase_replace(_Link_type const __N, _Link_type __q) throw ()
       {
         if (__q)
@@ -350,7 +414,7 @@ namespace KDTree
             _S_right(__q) = _S_right(__N);
             if (_S_right(__q)) _S_parent(_S_right(__q)) = __q;
           }
-        if (__N == _M_root()) _S_parent(_M_meta) = __q;
+        if (__N == _M_root()) _S_parent(_M_header) = __q;
         else if (_S_left(_S_parent(__N)) == __N)
           {
             _S_left(_S_parent(__N)) = __q;
@@ -366,7 +430,7 @@ namespace KDTree
         return __q;
       }
 
-      inline _Link_type
+      _Link_type
       _M_get_j_min(_Link_type const __N, size_t& __j,
                    size_t const __L) throw ()
       {
@@ -397,7 +461,7 @@ namespace KDTree
         return __ret;
       }
 
-      inline _Link_type
+      _Link_type
       _M_get_j_max(_Link_type const __N, size_t& __j,
                    size_t const __L) throw ()
       {
@@ -406,7 +470,7 @@ namespace KDTree
             __j = __L; 
             return __N; 
           }
-        _Node_compare __cmp(__j);
+        _Node_compare __cmp(__j % __K);
         _Link_type __ret = __N;
         if (_S_left(__N))
           {
@@ -428,7 +492,7 @@ namespace KDTree
         return __ret;
       }
 
-      inline void
+      void
       _M_erase_subtree(_Link_type __n)
       {
         while (__n)
@@ -440,7 +504,7 @@ namespace KDTree
           }
       }
 
-      inline const_iterator
+      const_iterator
       _M_find(_Link_type __N, const_reference __V,
               size_t const __L) const throw ()
       {
@@ -458,7 +522,7 @@ namespace KDTree
         return this->end();
       }
 
-      inline bool
+      bool
       _M_matches_node_in_d(_Link_type __N, const_reference __V,
                            size_t const __L) const throw ()
       {
@@ -466,7 +530,7 @@ namespace KDTree
         return !(__cmp(__N, __V) || __cmp(__V, __N));
       }
 
-      inline bool
+      bool
       _M_matches_node_in_other_ds(_Link_type __N, const_reference __V,
                                   size_t const __L = 0) const throw ()
       {
@@ -476,7 +540,7 @@ namespace KDTree
         return true;
       }
 
-      inline bool
+      bool
       _M_matches_node(_Link_type __N, const_reference __V,
                       size_t __L = 0) const throw ()
       {
@@ -484,24 +548,22 @@ namespace KDTree
           && _M_matches_node_in_other_ds(__N, __V, __L);
       }
 
-      template <typename _OutputIterator>
-        inline void
-        _M_find_within_range(_OutputIterator __out, size_t& __cnt,
-                             _Link_type __N, _Region const& __REGION,
+      size_t
+        _M_count_within_range(_Link_type __N, _Region const& __REGION,
                              _Region const& __BOUNDS,
                              size_t const __L) const throw ()
         {
+           size_t count = 0;
           if (__REGION.encloses(_S_value(__N)))
             {
-              *__out++ = _S_value(__N);
-              ++__cnt;
+               ++count;
             }
           if (_S_left(__N))
             {
               _Region __bounds(__BOUNDS);
               __bounds.set_high_bound(_S_value(__N), __L);
               if (__REGION.intersects_with(__bounds))
-                _M_find_within_range(__out, __cnt, _S_left(__N),
+                count += _M_count_within_range(_S_left(__N),
                                      __REGION, __bounds, __L+1);
             }
           if (_S_right(__N))
@@ -509,9 +571,43 @@ namespace KDTree
               _Region __bounds(__BOUNDS);
               __bounds.set_low_bound(_S_value(__N), __L);
               if (__REGION.intersects_with(__bounds))
-                _M_find_within_range(__out, __cnt, _S_right(__N),
+                count += _M_count_within_range(_S_right(__N),
                                      __REGION, __bounds, __L+1);
             }
+
+          return count;
+        }
+
+
+      template <typename _OutputIterator>
+        _OutputIterator
+        _M_find_within_range(_OutputIterator __out,
+                             _Link_type __N, _Region const& __REGION,
+                             _Region const& __BOUNDS,
+                             size_t const __L) const throw ()
+        {
+          if (__REGION.encloses(_S_value(__N)))
+            {
+              *__out++ = _S_value(__N);
+            }
+          if (_S_left(__N))
+            {
+              _Region __bounds(__BOUNDS);
+              __bounds.set_high_bound(_S_value(__N), __L);
+              if (__REGION.intersects_with(__bounds))
+                __out = _M_find_within_range(__out, _S_left(__N),
+                                     __REGION, __bounds, __L+1);
+            }
+          if (_S_right(__N))
+            {
+              _Region __bounds(__BOUNDS);
+              __bounds.set_low_bound(_S_value(__N), __L);
+              if (__REGION.intersects_with(__bounds))
+                __out = _M_find_within_range(__out, _S_right(__N),
+                                     __REGION, __bounds, __L+1);
+            }
+
+          return __out;
         }
 
       template <typename _Iter>
@@ -528,97 +624,97 @@ namespace KDTree
         if (++__m != __B) _M_optimise(__m, __B, __L+1);
       }
 
-      inline _Link_type&
+      _Link_type&
       _M_root() const 
       { 
-        return (_Link_type&) _M_meta->_M_parent; 
+        return (_Link_type&) _M_header->_M_parent; 
       }
       
-      inline _Link_type&
+      _Link_type&
       _M_leftmost() const 
       { 
-        return (_Link_type&) _M_meta->_M_left; 
+        return (_Link_type&) _M_header->_M_left; 
       }
       
-      inline _Link_type&
+      _Link_type&
       _M_rightmost() const 
       { 
-        return (_Link_type&) _M_meta->_M_right; 
+        return (_Link_type&) _M_header->_M_right; 
       }
 
-      static inline _Link_type&
+      static _Link_type&
       _S_parent(_Base_ptr N)
       {
         return (_Link_type&) N->_M_parent;
       }
 
-      static inline _Link_type&
+      static _Link_type&
       _S_parent(_Link_type N)
       {
         return (_Link_type&) N->_M_parent;
       }
 
-      static inline _Link_type&
+      static _Link_type&
       _S_left(_Base_ptr N)
       {
         return (_Link_type&) N->_M_left;
       }
 
-      static inline _Link_type&
+      static _Link_type&
       _S_left(_Link_type N)
       {
         return (_Link_type&) N->_M_left;
       }
 
-      static inline _Link_type&
+      static _Link_type&
       _S_right(_Base_ptr N)
       {
         return (_Link_type&) N->_M_right;
       }
 
-      static inline _Link_type&
+      static _Link_type&
       _S_right(_Link_type N)
       {
         return (_Link_type&) N->_M_right;
       }
 
-      static inline bool
+      static bool
       _S_is_leaf(_Base_ptr N)
       {
         return !_S_left(N) && !_S_right(N);
       }
 
-      static inline _Val
+      static _Val
       _S_value(_Link_type N)
       {
         return N->_M_value;
       }
 
-      static inline _Val
+      static _Val
       _S_value(_Base_ptr N)
       {
         return ((_Link_type) N)->_M_value;
       }
 
-      static inline _Link_type
+      static _Link_type
       _S_minimum(_Link_type const __X)
       {
         return (_Link_type) _Node_base::_S_minimum(__X);
       }
 
-      static inline _Link_type
+      static _Link_type
       _S_maximum(_Link_type const __X)
       {
         return (_Link_type) _Node_base::_S_maximum(__X);
       }
 
-      inline _Link_type
-      _M_new_node(const_reference __V = value_type(),
+      _Link_type
+      _M_new_node(const_reference __V, //  = value_type(),
                   _Base_ptr const __PARENT = NULL,
                   _Base_ptr const __LEFT = NULL,
                   _Base_ptr const __RIGHT = NULL)
       {
-        _Link_type __ret = _M_allocate_node();
+        _Link_type __ret = _Base::_M_allocate_node();
         try 
           {
             _M_construct_node(__ret, __V, __PARENT, __LEFT, __RIGHT);
@@ -631,7 +727,7 @@ namespace KDTree
         return __ret;
       }
 
-      inline _Link_type
+      _Link_type
       _M_clone_node(_Link_type const __X)
       {
         _Link_type __ret = _M_allocate_node(__X->_M_value);
@@ -639,14 +735,14 @@ namespace KDTree
         return __ret;
       }
 
-      inline void
+      void
       _M_delete_node(_Link_type __p)
       {
         _M_destroy_node(__p);
         _M_deallocate_node(__p);
       }
       
-      _Link_type _M_meta;
+      _Link_type _M_header;
       size_type _M_count;
 
 #ifdef KDTREE_DEFINE_OSTREAM_OPERATORS
@@ -657,43 +753,7 @@ namespace KDTree
 
   };
 
-#ifdef KDTREE_DEFINE_OSTREAM_OPERATORS
-
-  template <size_t const __K, typename _Val,
-            typename _Acc, typename _Cmp, typename _Alloc>
-    std::ostream&
-    operator<<(std::ostream& __out,
-               KDTree<__K, _Val, _Acc, _Cmp, _Alloc> const& __T) throw ()
-    {
-      __out << "meta node:   " << *__T._M_meta << std::endl;
-
-      if (__T.empty())
-        return __out << "[empty " << __K << "d-tree " << &__T << "]";
-
-      __out << "nodes total: " << __T.size() << std::endl;
-      __out << "dimensions:  " << __K << std::endl;
-
-      typedef KDTree<__K, _Val, _Acc, _Cmp, _Alloc> _Tree;
-      typedef typename _Tree::_Link_type _Link_type;
-
-      std::stack<_Link_type> s;
-      s.push(__T._M_root());
-      
-      while (!s.empty())
-        {
-          _Link_type n = s.top();
-          s.pop();
-          std::cout << *n << std::endl;
-          if (_Tree::_S_left(n)) s.push(_Tree::_S_left(n));
-          if (_Tree::_S_right(n)) s.push(_Tree::_S_right(n));
-        }
-
-      return __out;
-    }
-
-#endif
-
-}; // namespace KDTree
+} // namespace KDTree
 
 #endif // include guard
 
